package org.pelds.rules;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Class for holding rule instances, rules consist of a name and two lists of
 * atoms (predicates), the antecedent list and the consequent list.
 * 
 * @author Hannes Muehleisen (hannes@muehleisen.org)
 */
public class Rule {
	private String name;
	private ArrayList<RuleAtom> antAtoms = new ArrayList<RuleAtom>();
	private ArrayList<RuleAtom> conAtoms = new ArrayList<RuleAtom>();

	/**
	 * Get all variables used in this rule
	 * 
	 * @return List of variable names
	 */
	public Set<String> getUsedVariables() {
		HashSet<String> ret = new HashSet<String>();
		Iterator<RuleAtom> antIt = getAntAtoms().iterator();
		while (antIt.hasNext()) {
			ret.addAll(antIt.next().getVariableArguments());
		}
		Iterator<RuleAtom> conIt = getConAtoms().iterator();
		while (conIt.hasNext()) {
			ret.addAll(conIt.next().getVariableArguments());
		}
		return ret;
	}

	/**
	 * Convert this rule and its atoms to a String representation
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String ret = "";
		ret += getName() + ":\n";
		Iterator<RuleAtom> antIt = getAntAtoms().iterator();
		while (antIt.hasNext()) {
			ret += antIt.next().toString();
			if (antIt.hasNext())
				ret += " && ";
			ret += "\n";
		}
		ret += " => \n";
		Iterator<RuleAtom> conIt = getConAtoms().iterator();
		while (conIt.hasNext()) {
			ret += conIt.next().toString();
			if (conIt.hasNext())
				ret += " && \n";
		}
		ret += " ;\n";
		return ret;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public ArrayList<RuleAtom> getAntAtoms() {
		return antAtoms;
	}

	public void setAntAtoms(ArrayList<RuleAtom> antAtoms) {
		this.antAtoms = antAtoms;
	}

	public ArrayList<RuleAtom> getConAtoms() {
		return conAtoms;
	}

	public void setConAtoms(ArrayList<RuleAtom> conAtoms) {
		this.conAtoms = conAtoms;
	}

	public void addAntAtom(RuleAtom arg) {
		this.antAtoms.add(arg);
	}

	public void addConAtom(RuleAtom arg) {
		this.conAtoms.add(arg);
	}
}
