package org.pelds.rules;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Class for holding rule atom, atoms are predicates. Atoms are named by a
 * prefix and a name, as well as several arguments.
 * 
 * @author Hannes Muehleisen (hannes@muehleisen.org)
 */
public class RuleAtom {
	private String name;
	private ArrayList<String> arguments = new ArrayList<String>();
	private HashSet<String> variables = new HashSet<String>();

	/**
	 * Check this predicates arguments for variables indicated by their first
	 * character, "?"
	 * 
	 * @return a set of variable names
	 */
	public Set<String> getVariableArguments() {
		HashSet<String> ret = new HashSet<String>();
		Iterator<String> argIt = this.arguments.iterator();
		while (argIt.hasNext()) {
			String arg = argIt.next();
			if (arg.startsWith("?") && !ret.contains(arg))
				ret.add(arg);
		}
		return ret;
	}

	/**
	 * Convert this atom to a String representation
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String ret = "";
		ret += getName();
		ret += "(";
		Iterator<String> paramIt = getArguments().iterator();
		while (paramIt.hasNext()) {
			ret += paramIt.next();
			if (paramIt.hasNext())
				ret += ",";
		}
		ret += ")";
		return ret;
	}

	public HashSet<String> getVariables() {
		return variables;
	}

	public void setVariables(HashSet<String> variables) {
		this.variables = variables;
	}

	public void addVariable(String var) {
		this.variables.add(var);
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		if (!name.contains(":")) name = RuleQuery.PELDS_NS + name;
		this.name = name;
	}

	public ArrayList<String> getArguments() {
		return arguments;
	}

	public void setArguments(ArrayList<String> arguments) {
		this.arguments = arguments;
	}

	public void addArgument(String arg) {
		this.arguments.add(arg);
	}
}
