package org.pelds.rules;

import org.mindswap.pellet.jena.vocabulary.SWRL;
import org.mindswap.pellet.utils.Namespaces;

import com.hp.hpl.jena.ontology.ObjectProperty;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFList;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.vocabulary.RDF;

/**
 * <p>
 * Title:
 * </p>
 * <p>
 * Description:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2008
 * </p>

 * <p>
 * Company: Clark & Parsia, LLC. <http://www.clarkparsia.com>
 * </p>
 * 
 * @author Evren Sirin
 */
public class SWRLFactory {
	public static void main(String[] args) {
		OntModel model = ModelFactory.createOntologyModel( OntModelSpec.OWL_MEM );
		model.setNsPrefix( "swrl", Namespaces.SWRL );
		
		OntClass Male = model.createClass( "Male" );
		ObjectProperty hasParent = model.createObjectProperty( "hasParent" );
		ObjectProperty hasFather = model.createObjectProperty( "hasFather" );		
		
		SWRLFactory swrl = new SWRLFactory( model );
		
		Resource x = swrl.var( "x" );
		Resource y = swrl.var( "y" );
		
		swrl.rule( 
			swrl.body( swrl.individualPropertyAtom( hasParent, x, y ),
					   swrl.classAtom( Male, y ) ),
			swrl.head( swrl.individualPropertyAtom( hasFather, x, y ) ) );
		
		model.write( System.out, "TTL" );
	}

	private Model	model;

	public SWRLFactory(Model model) {
		this.model = model;
	}

	public RDFList body(Resource... atoms) {
		return list( atoms );
	}

	public Resource builtInAtom(Resource builtIn, RDFList args) {
		Resource atom = model.createResource();
		model.add( atom, RDF.type, SWRL.BuiltinAtom );
		model.add( atom, SWRL.builtin, builtIn );
		model.add( atom, SWRL.arguments, args );
		return atom;
	}

	public Resource classAtom(Resource cls, Resource arg) {
		Resource atom = model.createResource();
		model.add( atom, RDF.type, SWRL.ClassAtom );
		model.add( atom, SWRL.classPredicate, cls );
		model.add( atom, SWRL.argument1, arg );
		return atom;
	}

	public Resource dataPropertyAtom(Property prop, Resource arg1, RDFNode arg2) {
		Resource atom = model.createResource();
		model.add( atom, RDF.type, SWRL.DatavaluedPropertyAtom );
		model.add( atom, SWRL.propertyPredicate, prop );
		model.add( atom, SWRL.argument1, arg1 );
		model.add( atom, SWRL.argument2, arg2 );
		return atom;
	}

	public Resource dataRangeAtom(Resource dataRange, RDFNode arg) {
		Resource atom = model.createResource();
		model.add( atom, RDF.type, SWRL.DataRangeAtom );
		model.add( atom, SWRL.dataRange, dataRange );
		model.add( atom, SWRL.argument1, arg );
		return atom;
	}

	public Resource differentFrom(Resource arg1, Resource arg2) {
		Resource atom = model.createResource();
		model.add( atom, RDF.type, SWRL.DifferentIndividualsAtom );
		model.add( atom, SWRL.argument1, arg1 );
		model.add( atom, SWRL.argument2, arg2 );
		return atom;
	}

	public RDFList head(Resource... atoms) {
		return list( atoms );
	}

	public Resource individualPropertyAtom(Property prop, Resource arg1, Resource arg2) {
		Resource atom = model.createResource();
		model.add( atom, RDF.type, SWRL.IndividualPropertyAtom );
		model.add( atom, SWRL.propertyPredicate, prop );
		model.add( atom, SWRL.argument1, arg1 );
		model.add( atom, SWRL.argument2, arg2 );
		return atom;
	}

	public RDFList list(Resource... atoms) {
		return model.createList( atoms );
	}

	public Resource rule(RDFList body, RDFList head) {
		return rule( null, body, head );
	}

	public Resource rule(String uri, RDFList body, RDFList head) {
		Resource rule = uri == null
			? model.createResource()
			: model.createResource( uri );
		model.add( rule, RDF.type, SWRL.Imp );
		model.add( rule, SWRL.body, body );
		model.add( rule, SWRL.head, head );
		return rule;
	}
	
	public Resource sameAs(Resource arg1, Resource arg2) {
		Resource atom = model.createResource();
		model.add( atom, RDF.type, SWRL.SameIndividualAtom );
		model.add( atom, SWRL.argument1, arg1 );
		model.add( atom, SWRL.argument2, arg2 );
		return atom;
	}

	public Resource var(String name) {
		Resource var = model.createResource( name );
		model.add( var, RDF.type, SWRL.Variable );
		return var;
	}
}