package org.pelds.rules;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.GraphEvents;
import com.hp.hpl.jena.graph.GraphListener;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;

/**
 * UpdateGraphListener watches a graph, all added and deleted triples get added
 * to a list for inspection.
 * 
 * @author Hannes Muehleisen (hannes@muehleisen.org)
 */
public class UpdateGraphListener implements GraphListener {

	private Collection<Triple> originalTriples = new ArrayList<Triple>();

	private Collection<Triple> addedTriples = new ArrayList<Triple>();
	private Collection<Triple> deletedTriples = new ArrayList<Triple>();

	public UpdateGraphListener(Collection<Triple> someOriginalTriples) {
		originalTriples = someOriginalTriples;
	}

	public void notifyAddArray(Graph arg0, Triple[] arg1) {
		addedTriples.addAll(Arrays.asList(arg1));
	}

	public void notifyAddGraph(Graph arg0, Graph arg1) {
		addedTriples.addAll(arg1.find(null, null, null).toList());
	}

	public void notifyAddIterator(Graph arg0, Iterator<Triple> arg1) {
		while (arg1.hasNext())
			addedTriples.add(arg1.next());
	}

	public void notifyAddList(Graph arg0, List<Triple> arg1) {
		addedTriples.addAll(arg1);
	}

	public void notifyAddTriple(Graph arg0, Triple arg1) {
		addedTriples.add(arg1);
	}

	public void notifyDeleteArray(Graph arg0, Triple[] arg1) {
		deletedTriples.addAll(Arrays.asList(arg1));
	}

	public void notifyDeleteGraph(Graph arg0, Graph arg1) {
		deletedTriples.addAll(arg1.find(null, null, null).toList());
	}

	public void notifyDeleteIterator(Graph arg0, Iterator<Triple> arg1) {
		while (arg1.hasNext())
			deletedTriples.add(arg1.next());
	}

	public void notifyDeleteList(Graph arg0, List<Triple> arg1) {
		deletedTriples.addAll(arg1);
	}

	public void notifyDeleteTriple(Graph arg0, Triple arg1) {
		deletedTriples.add(arg1);
	}

	public void notifyEvent(Graph arg0, Object arg1) {
		if (arg1 instanceof GraphEvents
				&& ((GraphEvents) arg1).same(GraphEvents.removeAll)) {
			deletedTriples = originalTriples;
		}
	}

	public Collection<Triple> getChanges() {
		Collection<Triple> changedTriples = new ArrayList<Triple>();
		changedTriples.addAll(addedTriples);
		changedTriples.addAll(deletedTriples);
		return changedTriples;
	}

	public Model getChangesModel() {
		Model m = ModelFactory.createDefaultModel();
		Iterator<Triple> tIt = getChanges().iterator();
		while (tIt.hasNext())
			m.getGraph().add(tIt.next());
		return m;
	}

	public Collection<Triple> getAddedTriples() {
		return addedTriples;
	}

	public Collection<Triple> getDeletedTriples() {
		return deletedTriples;
	}

}