package org.pelds.servlet;

import java.net.MalformedURLException;
import java.net.URL;

import javax.servlet.http.HttpServletRequest;

/**
 * Prepares the request parameters for the certification request servlet
 * 
 * @author Hannes Muehleisen (hannes@muehleisen.org)
 * @see CertRequestServlet
 */
public class CertRequestParameters implements RequestParameters {
	// parameters
	protected int certLifetime;
	protected String certCN;
	protected URL certUrl;
	protected String certSpkac;

	// errors
	protected String errorMsgs;

	/**
	 * @see org.pelds.servlet.RequestParameters#process(javax.servlet.http.HttpServletRequest)
	 */
	public boolean process(HttpServletRequest req) {
		errorMsgs = "";
		try {
			certLifetime = Integer.parseInt(req.getParameter("lifetime"));
			if (certLifetime < 1) {
				errorMsgs += "Lifetime too short.";
			}
		} catch (Exception e) {
			errorMsgs += "lifetime not specified.";
		}
		try {
			certUrl = new URL(req.getParameter("url"));
		} catch (MalformedURLException e) {
			errorMsgs += "url (Web ID) malformed.";
		}
		certSpkac = req.getParameter("spkac");
		if (certSpkac == null || "".equals(certSpkac)) {
			errorMsgs += "SPKAC missing.";
		}
		return (errorMsgs.equals(""));
	}

	public String getErrorMsgs() {
		return errorMsgs;
	}

	public int getCertLifetime() {
		return certLifetime;
	}

	public String getCertCN() {
		return certCN;
	}

	public URL getCertUrl() {
		return certUrl;
	}

	public String getCertSpkac() {
		return certSpkac;
	}

}