package org.pelds.servlet;

import java.io.IOException;
import java.io.OutputStream;
import java.security.cert.X509Certificate;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.pelds.auth.CreateCert;

import com.hp.hpl.jena.sparql.util.Utils;

/**
 * Takes a SPKAC request, creates a SSL X509 certificate, sends that back to the
 * user and is happy
 * 
 * @author Hannes Muehleisen (hannes@muehleisen.org)
 */
public class CertRequestServlet extends Servlet {

	private static final long serialVersionUID = 1L;
	static private Logger log = Logger.getLogger("CertRequestServlet");

	/**
	 * @see org.pelds.servlet.Servlet#service(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	protected void service(HttpServletRequest req, HttpServletResponse resp) {
		log
				.info("Start processing certificate request " + req.hashCode()
						+ ".");

		// get (and check) the request parameters
		CertRequestParameters params = new CertRequestParameters();
		if (!params.process(req)) {
			log.info("BAD REQUEST for request " + req.hashCode() + ": "
					+ params.getErrorMsgs());
			try {
				resp.sendError(HttpServletResponse.SC_BAD_REQUEST, params
						.getErrorMsgs());
			} catch (IOException e) {
				log.warning("Sending the error reponse to request "
						+ req.hashCode() + " caused a " + Utils.className(e)
						+ ": " + e.getMessage());
			}
			return;
		}

		// create certificate builder instance
		CreateCert cert = new CreateCert(getConfig());
		byte[] certbytes = new byte[0];

		try {
			// create certificate
			X509Certificate clientcert = cert.createCert(params
					.getCertLifetime(), params.getCertUrl().toString(), params
					.getCertUrl(), params.getCertSpkac());
			certbytes = clientcert.getEncoded();
		} catch (Exception e1) {
			log.warning("Failed to create certificate: " + e1.getMessage());
			e1.printStackTrace();
			try {
				resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, e1
						.getMessage());
			} catch (IOException e) {
				log.warning("Sending the error reponse to request "
						+ req.hashCode() + " caused a " + Utils.className(e)
						+ ": " + e.getMessage());
			}
			return;
			// e1.printStackTrace();
		}
		// set headers
		resp.setHeader("Content-type", "application/x-x509-user-cert");
		resp.setHeader("Content-Length", new Integer(certbytes.length)
				.toString());
		try {
			// send to client
			OutputStream out = resp.getOutputStream();
			out.write(certbytes);
			out.flush();

		} catch (IOException e1) {
			log.warning("Failed to write to output stream: " + e1.getMessage());
			e1.printStackTrace();
		}

		// all okay!
		resp.setStatus(200);

		// finish
		try {
			resp.flushBuffer();
			log.info("Response buffer for request " + req.hashCode()
					+ " flushed.");
		} catch (IOException e) {
			log.warning("Flushing the response buffer for request "
					+ req.hashCode() + " caused a " + Utils.className(e) + ": "
					+ e.getMessage());
		}
		log.info("Finished processing request " + req.hashCode());

	}
}