package org.pelds.servlet;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.QueryException;
import com.hp.hpl.jena.sparql.modify.op.GraphUpdate1;
import com.hp.hpl.jena.sparql.modify.op.GraphUpdateN;
import com.hp.hpl.jena.sparql.modify.op.Update;
import com.hp.hpl.jena.update.UpdateFactory;
import com.hp.hpl.jena.update.UpdateRequest;
import javax.servlet.http.HttpServletRequest;

/**
 * Prepares the request parameters for the data update servlet
 * 
 * @author Hannes Muehleisen (hannes@muehleisen.org)
 * @see UpdateRequestServlet
 */
public class UpdateRequestParameters implements RequestParameters {
	// parameters
	protected String queryString;
	protected Collection<String> datasetUris;
	// parsed query
	protected UpdateRequest query;

	// errors
	protected String errorMsgs;

	/**
	 * @see org.pelds.servlet.RequestParameters#process(javax.servlet.http.HttpServletRequest)
	 */
	public boolean process(HttpServletRequest req) {
		errorMsgs = "";

		// the SPARQL query
		query = null;
		// the affected datasets URIs
		datasetUris = new ArrayList<String>();

		// force utf-8
		try {
			req.setCharacterEncoding("UTF-8");
		} catch (UnsupportedEncodingException e1) {
			// we'll just assume utf is well-known by 2009
		}

		queryString = req.getParameter("query");
		if (queryString == null || queryString.equals("")) {
			// fall back to "request" as in use by joseki a.o.
			queryString = req.getParameter("request");
		}
		if (queryString == null || queryString.equals("")) {
			errorMsgs += "Update Query not specified. ";
		} else {
			try {
				query = UpdateFactory.create(queryString);
				Iterator<Update> updateIt = query.iterator();
				while (updateIt.hasNext()) {
					Update up = updateIt.next();
					if (up instanceof GraphUpdate1) {
						datasetUris.add(((GraphUpdate1) up).getGraphName()
								.toString());
					}
					if (up instanceof GraphUpdateN) {
						List<Node> datasetNodes = ((GraphUpdateN) up)
								.getGraphNames();
						for (Node n : datasetNodes) {
							datasetUris.add(n.toString());
						}
					}
				}
			} catch (QueryException e) {
				errorMsgs += e.getMessage() + " ";
			}
		}
		if (datasetUris.size() == 0) { // nothing given in query, fall back to
										// query parameters
			// - the dataset URI
			String[] reqUris = req
			.getParameterValues("default-graph-uri");
			if (reqUris != null) {
				datasetUris = Arrays.asList(reqUris);
			}

			if (datasetUris.size() == 0) {
				errorMsgs += "Dataset not specified. ";
			}
		}
		return (errorMsgs.equals(""));
	}

	public String getErrorMsgs() {
		return errorMsgs;
	}

	public String getQueryString() {
		return queryString;
	}

	public UpdateRequest getQuery() {
		return query;
	}

	public Collection<String> getDatasetUris() {
		return datasetUris;
	}

}