package org.pelds.store;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import org.pelds.Config;
import org.pelds.rules.RuleException;

/**
 * Database access superclass, provides connection and configuration access
 * 
 * @author Hannes Muehleisen (hannes@muehleisen.org)
 */
public abstract class DbInterface {

	private static Connection connection = null;

	private static Config config = null;

	/**
	 * Get the JDBC connection, create if needed
	 * 
	 * @return a JDBC connection
	 */
	public Connection getConnection() {
		if (connection == null)
			createConnection();
		try {
			if (connection.isClosed()) {
				createConnection();
			}
		} catch (SQLException e) {
		}
		return connection;
	}

	/**
	 * Close the JDBC connection
	 */
	public void closeConnection() {
		/*
		 * if (connection != null) try { connection.close(); } catch
		 * (SQLException e) { } connection = null;
		 */
		// do nothing for now...
	}

	/**
	 * Opens a JDBC connection using to the configured access credentials
	 */
	private void createConnection() {
		String URL = config.getDbUrl();
		String USER = config.getDbUser();
		String PW = config.getDbPass();
		String CONN = config.getDbDriver();

		try {
			Class.forName(CONN);
		} catch (ClassNotFoundException e) {
			throw new RuleException("Could not find SQL connector " + CONN, e);
		}
		try {
			connection = DriverManager.getConnection(URL, USER, PW);
		} catch (SQLException e) {
			throw new RuleException(
					"Could not create SQL connection to " + URL, e);
		}
	}

	public void setConfig(Config c) {
		config = c;
	}

	public Config getConfig() {
		return config;
	}

}
