package org.pelds.store;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.pelds.rules.RuleException;

/**
 * API for access policy (rule) storage
 * 
 * @author Hannes Muehleisen (hannes@muehleisen.org)
 * 
 */
public class RuleStore extends DbInterface {
	private String datasetUri;

	/**
	 * Constructor, create a rule storage instance for the given graph
	 * 
	 * @param aDatasetUri
	 *            named graph identifier in form of an URI
	 */
	public RuleStore(String aDatasetUri) {
		datasetUri = aDatasetUri;
	}

	/**
	 * Get the current access policy in its String representation
	 * 
	 * @return access policy as String
	 */
	public String getRuleStr() {
		ResultSet rs = getRules();
		try {
			while (rs.next()) {
				return rs.getString("rule");
			}
		} catch (SQLException e) {
			throw new RuleException("SQL Error: " + e.getMessage());
		}
		return "";
	}

	/**
	 * Save a new access policy to the policy store
	 * 
	 * @param ruleStr
	 *            access policy as String
	 */
	public void setRuleStr(String ruleStr) {
		try {
			if (getRules().next()) { // update
				PreparedStatement ruleStmt = getConnection().prepareStatement(
						"UPDATE rules SET rule=? WHERE dataset=?");
				ruleStmt.setString(2, datasetUri);
				ruleStmt.setString(1, ruleStr);
				ruleStmt.executeUpdate();
			} else { // insert
				PreparedStatement ruleStmt = getConnection().prepareStatement(
						"INSERT INTO rules (dataset,rule) VALUES (?,?);");
				ruleStmt.setString(1, datasetUri);
				ruleStmt.setString(2, ruleStr);
				ruleStmt.executeUpdate();
			}

		} catch (SQLException e) {
			throw new RuleException("SQL Error: " + e.getMessage());

		}
	}

	/**
	 * Retrieve the access policy for the current dataset
	 * 
	 * @return SQL ResultSet containing rules in their String representation
	 */
	private ResultSet getRules() {
		try {
			PreparedStatement ruleStmt = getConnection().prepareStatement(
					"SELECT rule FROM rules WHERE dataset=?");
			ruleStmt.setString(1, datasetUri);
			return ruleStmt.executeQuery();
		} catch (SQLException e) {
			throw new RuleException("SQL Error: " + e.getMessage());
		}
	}
}
