package org.pelds.test;

import static org.junit.Assert.assertEquals;

import java.util.List;

import org.junit.Test;
import org.pelds.rules.Rule;
import org.pelds.rules.RuleParser;
import org.pelds.rules.RuleQuery;

public class RuleParserTests {

	// empty result with no rules
	@Test
	public void queryCommentNoRules() {
		List<Rule> rl = parse("/* JUST A COMMENT */");
		assertEquals(0,rl.size());
	}
	
	// no ending ;, should not work
	@Test
	public void queryNoSemiColonRules() {
		List<Rule> rl = parse("ARule: QueryAction(?a) => permit_triple(p://foo,*,*)");
		assertEquals(0,rl.size());
	}
	
	// no =>, should not work
	@Test
	public void queryNoArrowRules() {
		List<Rule> rl = parse("ARule: QueryAction(?a) permit_triple(p://foo,*,*);");
		assertEquals(0,rl.size());
	}
	
	// no antecedent, should work
	@Test
	public void queryNoAntRules() {
		List<Rule> rl = parse("ARule: => permit_triple(p://foo,*,*);");
		assertEquals(1,rl.size());
		assertEquals(1,rl.get(0).getConAtoms().size());
	}
	
	// no consequent, should not work
	@Test
	public void queryNoConRules() {
		List<Rule> rl = parse("ARule: QueryAction(?a) => ;");
		assertEquals(0,rl.size());
	}
	
	// no name,  should not work
	@Test
	public void queryNoNameRules() {
		List<Rule> rl = parse("QueryAction(?a) => permit_triple(p://foo,*,*);");
		assertEquals(0,rl.size());
	}

	// simple rule, only own ns
	@Test
	public void parseOneSimpleRule() {
		List<Rule> rl = parse
			("PublicRule: QueryAction(?a) => permit_triple(p://foo,*,*);");
		assertEquals(rl.size(),1);
		Rule r = rl.get(0);
		assertEquals("PublicRule",r.getName());
		
		assertEquals(1,r.getAntAtoms().size());
		assertEquals(1,r.getConAtoms().size());
		
		assertEquals(RuleQuery.PELDS_NS + "QueryAction",r.getAntAtoms().get(0).getName());
		assertEquals(1,r.getAntAtoms().get(0).getArguments().size());
		assertEquals("?a",r.getAntAtoms().get(0).getArguments().get(0));
		
		assertEquals(RuleQuery.PELDS_NS + "permit_triple",r.getConAtoms().get(0).getName());
		assertEquals(3,r.getConAtoms().get(0).getArguments().size());
		assertEquals("p://foo",r.getConAtoms().get(0).getArguments().get(0));
		assertEquals("*",r.getConAtoms().get(0).getArguments().get(1));
		assertEquals("*",r.getConAtoms().get(0).getArguments().get(2));
	}
	
	// multiple consequents / antecedents
	@Test
	public void queryMultiRules() {
		List<Rule> rl = parse("ARule: x:a(?b) && x:c(?d) => x:e(?f) && x:g(?h); BRule: x:i(?j) && x:k(?l) => x:m(?n) && x:o(?p) && x:q(?r,?s,?t);");
		assertEquals(2,rl.size());
		Rule a = rl.get(1);
		Rule b = rl.get(0);
		
		assertEquals(2,a.getAntAtoms().size());
		assertEquals(2,a.getConAtoms().size());
		
		assertEquals(2,b.getAntAtoms().size());
		assertEquals(3,b.getConAtoms().size());
	}
	
	
	
	private List<Rule> parse(String r) {
		RuleParser rp = new RuleParser (r,true);
		return rp.getRules();
	}
}