package org.pelds.test;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLEncoder;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import org.junit.Before;
import org.junit.Test;

import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.ResultSet;

public class ServletTests {

	private static final String SERVICE = "https://localhost:8443/pelds";
	private static final String SERVICE_QUERY = SERVICE + "/query";
	private static final String SERVICE_UPDATE = SERVICE + "/update";
	private static final int STATUS_OK = 200;
	private static final String PREFIX = "PREFIX : <p://exont> ";

	@Before
	public void prepareSockets() {
		HttpsURLConnection.setDefaultHostnameVerifier(hv);
		try {
			HttpsURLConnection.setDefaultSSLSocketFactory(getFactory(new File(
					getClass().getResource("testid-pw-asdf.p12").getPath()),
					"asdf"));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Test
	public void querySingleTriple() {
		// insert a triple
		assertEquals(STATUS_OK,
				update("INSERT INTO <p://graph1> { :r :p 123 }"));
		
		// check if its there
		ResultSet r1 = querySelect("SELECT ?v FROM <p://graph1> WHERE { :r :p ?v}");
		assertEquals(true, r1.hasNext());
		assertEquals("123^^http://www.w3.org/2001/XMLSchema#integer", r1.next()
				.get("v").toString());
		
		// remove the triple again
		assertEquals(STATUS_OK,
				update("DELETE FROM <p://graph1> { :r :p 123 }"));

		// check if its no longer there
		ResultSet r2 = querySelect("SELECT ?v FROM <p://graph1> WHERE { :r :p ?v}");
		assertEquals(false, r2.hasNext());
	}

	// helper
	private ResultSet querySelect(String q) {
		QueryExecution qe = QueryExecutionFactory.sparqlService(SERVICE_QUERY,
				PREFIX + q);
		return qe.execSelect();
	}

	private int update(String q) {
		try {
			HttpsURLConnection uc = getConnection(SERVICE_UPDATE + "?request="
					+ URLEncoder.encode(PREFIX + q, "utf8"));
			uc.connect();
			return uc.getResponseCode();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return -1;
	}

	private HttpsURLConnection getConnection(String url) throws IOException,
			UnrecoverableKeyException, KeyManagementException,
			NoSuchAlgorithmException, KeyStoreException, CertificateException {
		URL u = new URL(url);
		HttpsURLConnection uc = (HttpsURLConnection) u.openConnection();
		return uc;
	}

	private SSLSocketFactory getFactory(File pKeyFile, String pKeyPassword)
			throws NoSuchAlgorithmException, KeyStoreException,
			CertificateException, IOException, UnrecoverableKeyException,
			KeyManagementException {
		KeyManagerFactory keyManagerFactory = KeyManagerFactory
				.getInstance("SunX509");
		KeyStore keyStore = KeyStore.getInstance("PKCS12");

		InputStream keyInput = new FileInputStream(pKeyFile);
		keyStore.load(keyInput, pKeyPassword.toCharArray());
		keyInput.close();

		keyManagerFactory.init(keyStore, pKeyPassword.toCharArray());

		SSLContext context = SSLContext.getInstance("TLS");
		context.init(keyManagerFactory.getKeyManagers(), trustAllCerts,
				new SecureRandom());

		return context.getSocketFactory();
	}

	private static TrustManager[] trustAllCerts = new TrustManager[] { new X509TrustManager() {
		public java.security.cert.X509Certificate[] getAcceptedIssuers() {
			return null;
		}

		public void checkClientTrusted(
				java.security.cert.X509Certificate[] certs, String authType) {
		}

		public void checkServerTrusted(
				java.security.cert.X509Certificate[] certs, String authType) {
		}
	} };

	private static HostnameVerifier hv = new HostnameVerifier() {
		public boolean verify(String arg0, SSLSession arg1) {
			return true;
		}
	};
}
